@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend.c,v 1.162.2.23 2005/01/22 20:36:34 sniper Exp $ */
+/* $Id: zend.c,v 1.162.2.26.2.1 2005/06/09 10:14:25 dmitry Exp $ */
 
 #include "zend.h"
 #include "zend_extensions.h"
@@ -836,6 +836,7 @@ ZEND_API void zend_error(int type, const char *format, ...)
 			z_context->value.ht = EG(active_symbol_table);
 			z_context->type = IS_ARRAY;
 			ZVAL_ADDREF(z_context); /* we don't want this one to be freed */
+			z_context->is_ref = 1;
 
 			params = (zval ***) emalloc(sizeof(zval **)*5);
 			params[0] = &z_error_type;
@@ -847,6 +848,9 @@ ZEND_API void zend_error(int type, const char *format, ...)
 			orig_user_error_handler = EG(user_error_handler);
 			EG(user_error_handler) = NULL;
 			if (call_user_function_ex(CG(function_table), NULL, orig_user_error_handler, &retval, 5, params, 1, NULL TSRMLS_CC)==SUCCESS) {
+				if (Z_TYPE_P(z_context) != IS_ARRAY || z_context->value.ht != EG(active_symbol_table)) {
+					zend_error(E_ERROR, "User error handler must not modify error context");
+				}
 				if (retval) {
 					if (Z_TYPE_P(retval) == IS_BOOL && Z_LVAL_P(retval) == 0) {
 						zend_error_cb(type, error_filename, error_lineno, format, args);
@@ -857,15 +861,23 @@ ZEND_API void zend_error(int type, const char *format, ...)
 				/* The user error handler failed, use built-in error handler */
 				zend_error_cb(type, error_filename, error_lineno, format, args);
 			}
-			EG(user_error_handler) = orig_user_error_handler;
+			
+			if (!EG(user_error_handler)) {
+				EG(user_error_handler) = orig_user_error_handler;
+			} else {
+				zval_ptr_dtor(&orig_user_error_handler);
+			}
 
 			efree(params);
 			zval_ptr_dtor(&z_error_message);
 			zval_ptr_dtor(&z_error_type);
 			zval_ptr_dtor(&z_error_filename);
 			zval_ptr_dtor(&z_error_lineno);
-			if (ZVAL_REFCOUNT(z_context)==2) {
+			if (ZVAL_REFCOUNT(z_context) <= 2) {
 				FREE_ZVAL(z_context);
+			} else {
+				ZVAL_DELREF(z_context);
+				zval_ptr_dtor(&z_context);
 			}
 			break;
 	}
