@@ -326,7 +326,7 @@ void zend_do_unset(znode *variable TSRMLS_DC);
 void zend_do_isset_or_isempty(int type, znode *result, znode *variable TSRMLS_DC);
 
 void zend_do_foreach_begin(znode *foreach_token, znode *array, znode *open_brackets_token, znode *as_token, int variable TSRMLS_DC);
-void zend_do_foreach_cont(znode *value, znode *key, znode *as_token TSRMLS_DC);
+void zend_do_foreach_cont(znode *value, znode *key, znode *as_token, znode *foreach_token TSRMLS_DC);
 void zend_do_foreach_end(znode *foreach_token, znode *open_brackets_token TSRMLS_DC);
 
 void zend_do_declare_begin(TSRMLS_D);
@@ -532,6 +532,9 @@ int zendlex(znode *zendlval TSRMLS_DC);
 #define ZEND_FETCH_LOCAL	1
 #define ZEND_FETCH_STATIC	2
 
+/* variable parsing type (compile-time) */
+#define ZEND_PARSED_FCALL		(1<<0)
+
 /* var status for backpatching */
 #define BP_VAR_R			0
 #define BP_VAR_W			1
@@ -577,6 +580,9 @@ int zendlex(znode *zendlval TSRMLS_DC);
 #define ZEND_FETCH_STANDARD		0
 #define ZEND_FETCH_ADD_LOCK		1
 
+#define ZEND_FE_FETCH_BYREF	1
+#define ZEND_FE_FETCH_WITH_KEY	2
+
 #define ZEND_MEMBER_FUNC_CALL	1<<0
 #define ZEND_CTOR_CALL			1<<1
 
@@ -612,6 +618,8 @@ int zendlex(znode *zendlval TSRMLS_DC);
 #define ZEND_RETURN_VAL 0
 #define ZEND_RETURN_REF 1
 
+#define ZEND_RETURNS_FUNCTION 1<<0
+
 END_EXTERN_C()
 
 #endif /* ZEND_COMPILE_H */
