@@ -96,13 +96,13 @@ static void _zend_is_inconsistent(HashTable *ht, char *file, int line)
 	}
     switch (ht->inconsistent) {
 		case HT_IS_DESTROYING:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is being destroyed", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is being destroyed", file, line, ht);
 			break;
 		case HT_DESTROYED:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is already destroyed", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is already destroyed", file, line, ht);
 			break;
 		case HT_CLEANING:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is being cleaned", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is being cleaned", file, line, ht);
 			break;
     }
 	zend_bailout();
@@ -174,6 +174,7 @@ ZEND_API ulong zend_hash_func(char *arKey, uint nKeyLength)
 ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction, dtor_func_t pDestructor, int persistent)
 {
 	uint i = 3;
+	Bucket **tmp;
 
 	SET_INCONSISTENT(HT_OK);
 
@@ -183,14 +184,6 @@ ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction
 
 	ht->nTableSize = 1 << i;
 	ht->nTableMask = ht->nTableSize - 1;
-
-	/* Uses ecalloc() so that Bucket* == NULL */
-	ht->arBuckets = (Bucket **) pecalloc(ht->nTableSize, sizeof(Bucket *), persistent);
-
-	if (!ht->arBuckets) {
-		return FAILURE;
-	}
-
 	ht->pDestructor = pDestructor;
 	ht->pListHead = NULL;
 	ht->pListTail = NULL;
@@ -200,6 +193,16 @@ ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction
 	ht->persistent = persistent;
 	ht->nApplyCount = 0;
 	ht->bApplyProtection = 1;
+	ht->arBuckets = NULL;
+
+	/* Uses ecalloc() so that Bucket* == NULL */
+	tmp = (Bucket **) pecalloc(ht->nTableSize, sizeof(Bucket *), persistent);
+
+	if (!tmp) {
+		return FAILURE;
+	}
+	ht->arBuckets = tmp;
+
 	return SUCCESS;
 }
 
@@ -1063,7 +1066,7 @@ ZEND_API int zend_hash_get_current_key_ex(HashTable *ht, char **str_index, uint
 	if (p) {
 		if (p->nKeyLength) {
 			if (duplicate) {
-				*str_index = estrndup(p->arKey, p->nKeyLength);
+				*str_index = estrndup(p->arKey, p->nKeyLength-1);
 			} else {
 				*str_index = p->arKey;
 			}
@@ -1315,14 +1318,14 @@ void zend_hash_display(HashTable *ht)
 	for (i = 0; i < ht->nTableSize; i++) {
 		p = ht->arBuckets[i];
 		while (p != NULL) {
-			zend_output_debug_string(0, "%s <==> 0x%X\n", p->arKey, p->h);
+			zend_output_debug_string(0, "%s <==> 0x%lX\n", p->arKey, p->h);
 			p = p->pNext;
 		}
 	}
 
 	p = ht->pListTail;
 	while (p != NULL) {
-		zend_output_debug_string(0, "%s <==> 0x%X\n", p->arKey, p->h);
+		zend_output_debug_string(0, "%s <==> 0x%lX\n", p->arKey, p->h);
 		p = p->pListLast;
 	}
 }
