@@ -69,17 +69,9 @@ ZEND_API void zend_html_puts(const char *s, uint len TSRMLS_DC)
 	
 	while (ptr<end) {
 		if (*ptr==' ') {
-			/* Series of spaces should be displayed as &nbsp;'s
-			 * whereas single spaces should be displayed as a space
-			 */
-			if ((ptr+1) < end && *(ptr+1)==' ') {
-				do {
-					zend_html_putc(*ptr);
-				} while ((++ptr < end) && (*ptr==' '));
-			} else {
-				(void) ZEND_PUTC(*ptr);
-				ptr++;
-			}
+			do {
+				zend_html_putc(*ptr);
+			} while ((++ptr < end) && (*ptr==' '));
 		} else {
 			zend_html_putc(*ptr++);
 		}
