@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_ftp.c,v 1.74.2.11 2003/08/07 16:03:05 zeev Exp $ */
+/* $Id: php_ftp.c,v 1.74.2.15 2005/03/10 23:38:18 iliaa Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -602,16 +602,20 @@ PHP_FUNCTION(ftp_get)
 		RETURN_FALSE;
 	}
 
+#ifdef PHP_WIN32
+	mode = FTPTYPE_IMAGE;
+#endif
+
 	if (ftp->autoseek && resumepos) {
-		if (PG(safe_mode) && (!php_checkuid(local, "rb+", CHECKUID_CHECK_MODE_PARAM))) {
+		if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "rt+" : "rb+", CHECKUID_CHECK_MODE_PARAM))) {
 			RETURN_FALSE;
 		}
-		outstream = php_stream_fopen(local, "rb+", NULL);
+		outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "rt+" : "rb+", NULL);
 		if (outstream == NULL) {
-			if (PG(safe_mode) && (!php_checkuid(local, "wb", CHECKUID_CHECK_MODE_PARAM))) {
+			if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "wt" : "wb", CHECKUID_CHECK_MODE_PARAM))) {
 				RETURN_FALSE;
 			}
-			outstream = php_stream_fopen(local, "wb", NULL);
+			outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "wt" : "wb", NULL);
 		}
 		if (outstream != NULL) {
 			/* if autoresume is wanted seek to end */
@@ -623,10 +627,10 @@ PHP_FUNCTION(ftp_get)
 			}
 		}
 	} else {
-		if (PG(safe_mode) && (!php_checkuid(local, "wb", CHECKUID_CHECK_MODE_PARAM))) {
+		if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "wt" : "wb", CHECKUID_CHECK_MODE_PARAM))) {
 			RETURN_FALSE;
 		}
-		outstream = php_stream_fopen(local, "wb", NULL);
+		outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "wt" : "wb", NULL);
 	}
 
 	if (outstream == NULL)	{
@@ -636,6 +640,7 @@ PHP_FUNCTION(ftp_get)
 
 	if (!ftp_get(ftp, outstream, remote, xtype, resumepos)) {
 		php_stream_close(outstream);
+		VCWD_UNLINK(local);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", ftp->inbuf);
 		RETURN_FALSE;
 	}
@@ -674,15 +679,15 @@ PHP_FUNCTION(ftp_nb_get)
 	}
 
 	if (ftp->autoseek && resumepos) {
-		if (PG(safe_mode) && (!php_checkuid(local, "rb+", CHECKUID_CHECK_MODE_PARAM))) {
+		if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "rt+" : "rb+", CHECKUID_CHECK_MODE_PARAM))) {
 			RETURN_FALSE;
 		}
-		outstream = php_stream_fopen(local, "rb+", NULL);
+		outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "rt+" : "rb+", NULL);
 		if (outstream == NULL) {
-			if (PG(safe_mode) && (!php_checkuid(local, "wb", CHECKUID_CHECK_MODE_PARAM))) {
+			if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "wt" : "wb", CHECKUID_CHECK_MODE_PARAM))) {
 				RETURN_FALSE;
 			}
-			outstream = php_stream_fopen(local, "wb", NULL);
+			outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "wt" : "wb", NULL);
 		}
 		if (outstream != NULL) {
 			/* if autoresume is wanted seek to end */
@@ -694,10 +699,10 @@ PHP_FUNCTION(ftp_nb_get)
 			}
 		}
 	} else {
-		if (PG(safe_mode) && (!php_checkuid(local, "wb", CHECKUID_CHECK_MODE_PARAM))) {
+		if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "wt" : "wb", CHECKUID_CHECK_MODE_PARAM))) {
 			RETURN_FALSE;
 		}
-		outstream = php_stream_fopen(local, "wb", NULL);
+		outstream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "wt" : "wb", NULL);
 	}
 
 	if (outstream == NULL)	{
@@ -711,6 +716,7 @@ PHP_FUNCTION(ftp_nb_get)
 
 	if ((ret = ftp_nb_get(ftp, outstream, remote, xtype, resumepos TSRMLS_CC)) == PHP_FTP_FAILED) {
 		php_stream_close(outstream);
+		VCWD_UNLINK(local);
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", ftp->inbuf);
 		RETURN_LONG(PHP_FTP_FAILED);
 	}
@@ -881,11 +887,11 @@ PHP_FUNCTION(ftp_put)
 	if (php_check_open_basedir(local TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
-	if (PG(safe_mode) && (!php_checkuid(local, "rb", CHECKUID_CHECK_MODE_PARAM))) {
+	if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "rt" : "rb", CHECKUID_CHECK_MODE_PARAM))) {
 		RETURN_FALSE;
 	}
 
-	instream = php_stream_fopen(local, "rb", NULL);
+	instream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "rt" : "rb", NULL);
 
 	if (instream == NULL)	{
 		RETURN_FALSE;
@@ -943,11 +949,11 @@ PHP_FUNCTION(ftp_nb_put)
 	if (php_check_open_basedir(local TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
-	if (PG(safe_mode) && (!php_checkuid(local, "rb", CHECKUID_CHECK_MODE_PARAM))) {
+	if (PG(safe_mode) && (!php_checkuid(local, mode == FTPTYPE_ASCII ? "rt" : "rb", CHECKUID_CHECK_MODE_PARAM))) {
 		RETURN_FALSE;
 	}
 
-	instream = php_stream_fopen(local, "rb", NULL);
+	instream = php_stream_fopen(local, mode == FTPTYPE_ASCII ? "rt" : "rb", NULL);
 
 	if (instream == NULL)	{
 		RETURN_FALSE;
@@ -1161,7 +1167,7 @@ PHP_FUNCTION(ftp_set_option)
 			RETURN_TRUE;
 			break;
 		default:
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown option '%d'", option);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown option '%ld'", option);
 			RETURN_FALSE;
 			break;
 	}
@@ -1190,7 +1196,7 @@ PHP_FUNCTION(ftp_get_option)
 			RETURN_BOOL(ftp->autoseek);
 			break;
 		default:
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown option '%d'", option);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown option '%ld'", option);
 			RETURN_FALSE;
 			break;
 	}
