@@ -6,7 +6,7 @@ This file is public domain and comes with NO WARRANTY of any kind */
 #include <global.h>
 #include <m_string.h>
 #if !defined(HAVE_STRTOULL) && defined(HAVE_LONG_LONG)
-#define UNSIGNED
-#define MYSQL_LONGLONG
+#define USE_UNSIGNED
+#define USE_LONGLONG
 #include "strto.c"
 #endif
