@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2002 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
  
-/* $Id: php_pgsql.h,v 1.56 2002/10/17 23:41:34 yohgaki Exp $ */
+/* $Id: php_pgsql.h,v 1.56.2.4 2005/04/13 21:27:35 derick Exp $ */
 
 #ifndef PHP_PGSQL_H
 #define PHP_PGSQL_H
@@ -83,7 +83,9 @@ PHP_FUNCTION(pg_fetch_object);
 PHP_FUNCTION(pg_fetch_result);
 PHP_FUNCTION(pg_fetch_row);
 PHP_FUNCTION(pg_fetch_all);
+#if HAVE_PQCMDTUPLES
 PHP_FUNCTION(pg_affected_rows);
+#endif
 PHP_FUNCTION(pg_get_result);
 PHP_FUNCTION(pg_result_seek);
 PHP_FUNCTION(pg_result_status);
@@ -196,6 +198,7 @@ typedef enum _php_pgsql_data_type {
 	PG_DATE,
 	PG_TIME,
 	PG_TIME_WITH_TIMEZONE,
+	PG_TIMESTAMP,
 	PG_TIMESTAMP_WITH_TIMEZONE,
 	PG_INTERVAL,
 	/* binary */
@@ -235,24 +238,23 @@ typedef struct _php_pgsql_notice {
 	size_t len;
 } php_pgsql_notice;
 
-typedef struct {
+ZEND_BEGIN_MODULE_GLOBALS(pgsql)
 	long default_link; /* default link when connection is omitted */
 	long num_links,num_persistent;
 	long max_links,max_persistent;
 	long allow_persistent;
 	long auto_reset_persistent;
 	int le_lofp,le_string;
-	int ignore_notices,log_notices;
+	long ignore_notices,log_notices;
 	HashTable notices;  /* notice message for each connection */
-} php_pgsql_globals;
+ZEND_END_MODULE_GLOBALS(pgsql)
 
+ZEND_EXTERN_MODULE_GLOBALS(pgsql)
 
 #ifdef ZTS
-# define PGG(v) TSRMG(pgsql_globals_id, php_pgsql_globals *, v)
-extern int pgsql_globals_id;
+# define PGG(v) TSRMG(pgsql_globals_id, zend_pgsql_globals *, v)
 #else
 # define PGG(v) (pgsql_globals.v)
-extern php_pgsql_globals pgsql_globals;
 #endif
 
 #endif
