@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2002 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,17 +16,18 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_mail.h,v 1.12 2002/02/28 08:26:47 sebastian Exp $ */
+/* $Id: php_mail.h,v 1.14.4.1 2002/12/31 16:35:33 sebastian Exp $ */
 
 #ifndef PHP_MAIL_H
 #define PHP_MAIL_H
 
+PHP_FUNCTION(mail);
+PHP_MINFO_FUNCTION(mail);
+
 #if HAVE_SENDMAIL
 
-PHP_FUNCTION(mail);
 PHP_FUNCTION(ezmlm_hash);
-PHP_MINFO_FUNCTION(mail);
-PHPAPI extern int php_mail(char *to, char *subject, char *message, char *headers, char *extra_cmd);
+PHPAPI extern int php_mail(char *to, char *subject, char *message, char *headers, char *extra_cmd TSRMLS_DC);
 
 #endif
 
