@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2003 The PHP Group                                |
+   | Copyright (c) 1997-2004 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
+   | This source file is subject to version 3.0 of the PHP license,       |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_0.txt.                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_sybase_ct.h,v 1.14.2.1 2002/12/31 16:35:38 sebastian Exp $ */
+/* $Id: php_sybase_ct.h,v 1.14.2.3 2004/01/25 20:14:05 thekid Exp $ */
 
 #ifndef PHP_SYBASE_CT_H
 #define PHP_SYBASE_CT_H
@@ -65,6 +65,7 @@ ZEND_BEGIN_MODULE_GLOBALS(sybase)
 	long default_link;
 	long num_links,num_persistent;
 	long max_links,max_persistent;
+	long login_timeout;
 	long allow_persistent;
 	char *appname;
 	char *hostname;
@@ -83,6 +84,7 @@ typedef struct {
 	int dead;
 	int active_result_index;
 	long affected_rows;
+	zval *callback_name;
 } sybase_link;
 
 #define SYBASE_ROWS_BLOCK 128
