@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2002 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_yaz.h,v 1.13 2002/07/12 11:45:04 sniper Exp $ */
+/* $Id: php_yaz.h,v 1.13.4.2 2003/03/10 13:35:21 ddhill Exp $ */
 
 #ifndef PHP_YAZ_H
 #define PHP_YAZ_H
@@ -55,7 +55,7 @@ PHP_FUNCTION(yaz_schema);
 
 ZEND_BEGIN_MODULE_GLOBALS(yaz)
     int assoc_seq;
-    int max_links;
+    long max_links;
     char *log_file;
 ZEND_END_MODULE_GLOBALS(yaz)
     
