@@ -16,7 +16,7 @@
    | Streams work by Wez Furlong <wez@thebrainroom.com>                   |
    +----------------------------------------------------------------------+
  */
-/* $Id: network.c,v 1.83.2.25 2004/05/23 10:43:15 wez Exp $ */
+/* $Id: network.c,v 1.83.2.27 2005/01/04 10:35:56 sniper Exp $ */
 
 /*#define DEBUG_MAIN_NETWORK 1*/
 
@@ -430,7 +430,7 @@ PHPAPI int php_connect_nonb_win32(SOCKET sockfd,
  * port, returns the created socket on success, else returns -1.
  * timeout gives timeout in seconds, 0 means blocking mode.
  */
-int php_hostconnect(const char *host, unsigned short port, int socktype, struct timeval *timeout TSRMLS_DC)
+PHPAPI int php_hostconnect(const char *host, unsigned short port, int socktype, struct timeval *timeout TSRMLS_DC)
 {	
 	int n, repeatto, s;
 	struct sockaddr **sal, **psal;
@@ -704,8 +704,11 @@ PHPAPI php_stream *_php_stream_sock_open_unix(const char *path, int pathlen, con
 	
 	memcpy(unix_addr.sun_path, path, pathlen);
 
-	if (php_connect_nonb(socketd, (struct sockaddr *) &unix_addr, sizeof(unix_addr), timeout) == SOCK_CONN_ERR) 
+	if (php_connect_nonb(socketd, (struct sockaddr *) &unix_addr,
+			sizeof(unix_addr), timeout) == SOCK_CONN_ERR) {
+		closesocket(socketd);
 		return NULL;
+	}
 
 	stream = php_stream_sock_open_from_socket_rel(socketd, persistent_id);
 	if (stream == NULL)
