@@ -12,9 +12,11 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Author: Stig Sther Bakken <ssb@fast.no>                             |
+   | Author: Stig Sther Bakken <ssb@php.net>                             |
    +----------------------------------------------------------------------+
- */
+*/
+
+/* $Id: snprintf.h,v 1.18.4.7 2005/04/08 05:44:53 sniper Exp $ */
 
 /*
 
@@ -62,31 +64,61 @@ Example:
 #ifndef SNPRINTF_H
 #define SNPRINTF_H
 
-#if !defined(HAVE_SNPRINTF) || PHP_BROKEN_SNPRINTF
-extern int ap_php_snprintf(char *, size_t, const char *, ...);
+BEGIN_EXTERN_C()
+PHPAPI int ap_php_snprintf(char *, size_t, const char *, ...) PHP_ATTRIBUTE_FORMAT(printf, 3, 4);
+PHPAPI int ap_php_vsnprintf(char *, size_t, const char *, va_list ap) PHP_ATTRIBUTE_FORMAT(printf, 3, 0);
+PHPAPI int php_sprintf (char* s, const char* format, ...) PHP_ATTRIBUTE_FORMAT(printf, 2, 3);
+END_EXTERN_C()
+
+#ifdef snprintf
+#undef snprintf
+#endif
 #define snprintf ap_php_snprintf
-#endif
 
-#if !defined(HAVE_VSNPRINTF) || PHP_BROKEN_VSNPRINTF
-extern int ap_php_vsnprintf(char *, size_t, const char *, va_list ap);
+#ifdef vsnprintf
+#undef vsnprintf
+#endif
 #define vsnprintf ap_php_vsnprintf
-#endif
 
-#if PHP_BROKEN_SPRINTF
-int php_sprintf (char* s, const char* format, ...);
-#define sprintf php_sprintf
+#ifdef sprintf
+#undef sprintf
 #endif
+#define sprintf php_sprintf
 
 typedef enum {
 	NO = 0, YES = 1
 } boolean_e;
 
+typedef enum {
+	LM_STD = 0,
+#if SIZEOF_INTMAX_T
+	LM_INTMAX_T,
+#endif
+#if SIZEOF_PTRDIFF_T
+	LM_PTRDIFF_T,
+#endif
+#if SIZEOF_LONG_LONG
+	LM_LONG_LONG,
+#endif
+	LM_SIZE_T,
+	LM_LONG,
+	LM_LONG_DOUBLE
+} length_modifier_e;
+
 extern char * ap_php_cvt(double arg, int ndigits, int *decpt, int *sign, int eflag, char *buf);
 extern char * ap_php_ecvt(double arg, int ndigits, int *decpt, int *sign, char *buf);
 extern char * ap_php_fcvt(double arg, int ndigits, int *decpt, int *sign, char *buf);
 extern char * ap_php_gcvt(double number, int ndigit, char *buf, boolean_e altform);
 
-#define WIDE_INT		long
+#if SIZEOF_LONG_LONG_INT
+# define WIDE_INT		long long int
+#elif SIZEOF_LONG_LONG
+# define WIDE_INT		long long
+#elif _WIN64
+# define WIDE_INT		__int64
+#else
+# define WIDE_INT		long
+#endif
 typedef WIDE_INT wide_int;
 typedef unsigned WIDE_INT u_wide_int;
 
