@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: sapi_apache2.c,v 1.91.2.26 2005/01/07 06:28:36 sniper Exp $ */
+/* $Id: sapi_apache2.c,v 1.91.2.27 2005/04/08 20:35:02 sniper Exp $ */
 
 #include <fcntl.h>
 
@@ -267,18 +267,11 @@ static void php_apache_sapi_log_message(char *msg)
 
 	ctx = SG(server_context);
    
-	/* We use APLOG_STARTUP because it keeps us from printing the
-	 * data and time information at the beginning of the error log
-	 * line.  Not sure if this is correct, but it mirrors what happens
-	 * with Apache 1.3 -- rbb
-	 */
 	if (ctx == NULL) { /* we haven't initialized our ctx yet, oh well */
-		ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO | APLOG_STARTUP,
-					 0, NULL, "%s", msg);
+		ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_STARTUP, 0, NULL, "%s", msg);
 	}
 	else {
-		ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO | APLOG_STARTUP,
-					 0, ctx->r->server, "%s", msg);
+		ap_log_error(APLOG_MARK, APLOG_ERR, 0, ctx->r->server, "%s", msg);
 	}
 }
 
@@ -356,7 +349,7 @@ static int php_input_filter(ap_filter_t *f, apr_bucket_brigade *bb,
 
 	ctx = SG(server_context);
 	if (ctx == NULL) {
-		ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, 0, f->r,
+		ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, f->r,
 					 "php failed to get server context");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
@@ -467,7 +460,7 @@ static int php_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
 	
 	ctx = SG(server_context);
 	if (ctx == NULL) {
-		ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, 0, f->r,
+		ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, f->r,
 					 "php failed to get server context");
 		zend_try {
 			zend_ini_deactivate(TSRMLS_C);
@@ -627,7 +620,7 @@ static void php_add_filter(request_rec *r, ap_filter_t *f)
 	/* for those who still have Set*Filter PHP configured */
 	while (f) {
 		if (strcmp(f->frec->name, "PHP") == 0) {
-			ap_log_error(APLOG_MARK, APLOG_WARNING | APLOG_NOERRNO,
+			ap_log_error(APLOG_MARK, APLOG_WARNING,
 				     0, r->server,
 				     "\"Set%sFilter PHP\" already configured for %s",
 				     output ? "Output" : "Input", r->uri);
