@@ -20,7 +20,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: cgi_main.c,v 1.190.2.60 2004/03/31 17:01:45 iliaa Exp $ */
+/* $Id: cgi_main.c,v 1.190.2.68 2005/04/28 14:24:47 sniper Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -98,7 +98,7 @@ static void (*php_php_import_environment_variables)(zval *array_ptr TSRMLS_DC);
 /**
  * Number of child processes that will get created to service requests
  */
-static int children = 8;
+static int children = 0;
 
 /**
  * Set to non-zero if we are the parent process
@@ -177,7 +177,7 @@ static int module_name_cmp(const void *a, const void *b TSRMLS_DC)
 	Bucket *f = *((Bucket **) a);
 	Bucket *s = *((Bucket **) b);
 
-	return strcmp(((zend_module_entry *)f->pData)->name,
+	return strcasecmp(((zend_module_entry *)f->pData)->name,
 				  ((zend_module_entry *)s->pData)->name);
 }
 
@@ -481,6 +481,20 @@ static void sapi_cgi_register_variables(zval *track_vars_array TSRMLS_DC)
 
 static void sapi_cgi_log_message(char *message)
 {
+#if PHP_FASTCGI
+	long logging = 1;
+	TSRMLS_FETCH();
+
+	if (cfg_get_long("fastcgi.logging", &logging) == FAILURE) {
+		logging = 1;
+	}
+                                                                                                        
+	if (!FCGX_IsCGI() && logging) {
+		FCGX_Request *request = (FCGX_Request *)SG(server_context);
+		FCGX_FPrintF( request->err, "%s\n", message );
+		/* ignore return code */
+	} else
+#endif /* PHP_FASTCGI */
 	fprintf(stderr, "%s\n", message);
 }
 
@@ -989,7 +1003,7 @@ int main(int argc, char *argv[])
 	int fcgi_fd = 0;
 	FCGX_Request request;
 #ifdef PHP_WIN32
-	int impersonate = 0;
+	long impersonate = 0;
 #else
     int status = 0;
 #endif
